import json
import time


class JIMResponse:
    # на будущее
    response_ids = {'100': 'Basic Notification', '101': 'Important Notification',
                    '200': 'OK', '201': 'Object Created', '202': 'Accepted',
                    '400': 'Malformed JSON', '401': 'Not Authorised', '402': 'Wrong Login/Password',
                    '403': 'Forbidden', '404': 'Not Found', '409': 'Connection Conflict',
                    '410': 'User Gone', '500': 'Server Error',
                    }

    def __init__(self, code, text='', resp_time=None, quantity=None, uid=''):
        self.response = str(code)  # можно передавать число или строку
        if quantity:  # Возврат количества контактов клиенту
            self.quantity = str(quantity)
            return
        if resp_time is None:  # когда клиент читает ответ сервера, у него уже есть отметка времени
            self.time = time.ctime(time.time())
        else:
            self.time = resp_time
        if self.response[0] == '1' or self.response[0] == '2':
            self.alert = text
        else:
            self.error = text
        self.uid = uid

    @property
    def json(self):
        fields = self.__dict__
        j = {c: fields[c] for c in fields if fields[c] is not None}
        return json.dumps(j)

    @property
    def utf8(self):
        return self.json.encode('utf-8')

    @staticmethod
    def fromjson(json_resp):
        try:
            parsed_resp = json.loads(json_resp)
        except ValueError:
            return None
        if 'alert' in parsed_resp or 'error' in parsed_resp:
            text = parsed_resp['alert'] if 'alert' in parsed_resp else parsed_resp['error']
        else:
            text = None
        time = parsed_resp['time'] if 'time' in parsed_resp else None
        quantity = parsed_resp['quantity'] if 'quantity' in parsed_resp else None
        uid = parsed_resp['uid'] if 'uid' in parsed_resp else None
        response = JIMResponse(parsed_resp['response'], text, time, quantity, uid)
        return response

if __name__ == '__main__':
    resp = JIMResponse(300, 'Hello there')
    print(resp.json)
    print(resp.quantity)
    time.sleep(5)
    resp2 = JIMResponse.fromjson(resp.json)
    print(resp2.json)
    resp3 = JIMResponse(202, quantity=200)
    print(resp3.json)



import json
import time

from aemessenger.Server.serverstorage import ContactList, ServerDB, Client

from aemessenger.JIM.jimresponse import JIMResponse
from aemessenger.JIM.jimmsg import JIMSendContactListMsg


class JIMServerController(object):
    def __init__(self):
        self.db = ServerDB()

    def parse_client_message(self, msg_bytes, from_username):
        timestr = time.ctime(time.time())
        # предварительно найти клиента в базе
        owner = None
        if from_username:
            owner = self.db.session.query(Client).filter_by(login=from_username).first()
        try:
            parsed_msg = json.loads(msg_bytes)
        except ValueError:
            print(msg_bytes)
            return JIMResponse(400, 'Incorrectly formed JSON', timestr, None, None), \
                   ServerAction('error', None, from_username)
        if 'action' in parsed_msg:
            action = parsed_msg['action']

            if action == 'presence':
                serv_action = ServerAction('presence', None, (parsed_msg['user']['account_name'],))
                return JIMResponse(200, 'OK', None, None, parsed_msg['uid']), serv_action

            elif action == 'msg':
                to = parsed_msg['to']
                if to.startswith('#'):
                    pass
                else:
                    serv_action = ServerAction('msg', msg_bytes, (to,))
                    response = JIMResponse(200, 'OK', timestr, None, parsed_msg['uid'])
                    return response, serv_action

            elif action == 'get_contacts':
                if owner:
                    # Найти контакты в базе и получить их id
                    contacts_query = self.db.session.query(ContactList).filter_by(ownerid=owner.id).all()
                    contacts_ids = [x.contactid for x in contacts_query]
                    # Получить объекты Client из id контактов
                    contacts = self.db.session.query(Client).filter(Client.id.in_(contacts_ids)).all()
                    serv_action = ServerAction('quantity', len(contacts), (from_username,))
                    responses = [JIMResponse(202, None, timestr, len(contacts), parsed_msg['uid'])]
                    for contact in contacts:
                        responses.append(JIMSendContactListMsg(contact.login))
                    return responses, serv_action
                else:
                    response = JIMResponse(404, 'User not found', timestr, None, parsed_msg['uid'])
                    return response, None

            elif action == 'add_contact':
                if owner:
                    # Найдем контакт в базе
                    user_query = self.db.session.query(Client).filter_by(login=parsed_msg['contact_id']).first()
                    if not user_query:
                        return JIMResponse(404, 'Contact not found', timestr, None, parsed_msg['uid']), None
                    # Проверим, что его нет в контакт листе клиента
                    contacts_query = self.db.session.query(ContactList)\
                        .filter_by(ownerid=owner.id)\
                        .filter_by(contactid=user_query.id)\
                        .first()
                    if contacts_query:  # контакт уже есть в базе данных
                        return JIMResponse(100, 'Contact already exists', timestr, None, parsed_msg['uid']), None
                    # Добавим контакт в контакт лист клиенту
                    contact = ContactList(owner.id, user_query.id)
                    self.db.session.add(contact)
                    self.db.session.commit()
                    return JIMResponse(200, 'OK', timestr, None, parsed_msg['uid']), None
                else:
                    response = JIMResponse(404, 'Please log in.', timestr, None, parsed_msg['uid']), None
                    return response, None

            elif action == 'del_contact':
                if owner:
                    user_query = self.db.session.query(Client).filter_by(login=parsed_msg['contact_id']).first()
                    if not user_query:
                        return JIMResponse(404, 'Contact not found', timestr, None, parsed_msg['uid']), None
                    self.db.session.delete(user_query)
                    self.db.session.commit()
                    return JIMResponse(200, 'OK', timestr, None, parsed_msg['uid']), None
                else:
                    response = JIMResponse(404, 'Please log in.', timestr, None, parsed_msg['uid'])
                    return response, None
        else:
            return JIMResponse(400, 'Incorrectly formed JSON', timestr, None, parsed_msg['uid']), None

    def get_server_response(self, response, alert, error):  # TODO: СТАРЫЙ КОД, УБРАТЬ
        msg = ''
        timestr = time.ctime(time.time())
        if alert:
            msg = json.dumps({'response': response, 'time': timestr, 'alert': alert})
        if error:
            msg = json.dumps({'response': response, 'time': timestr, 'error': error})
        return msg


class ServerAction(object):
    def __init__(self, action, msg, usernames):
        self.action = action  # Что делать
        self.msg = msg  # Как делать
        self.usernames = usernames  # Кому делать



import logging
import os
from functools import wraps
from logging.handlers import TimedRotatingFileHandler


def configlogging(isserver: bool):
    package_dir = os.path.abspath(os.path.dirname(__file__))
    fmt = '%(asctime)s %(levelname)s %(message)s'
    if isserver:
        filename = os.path.join(package_dir, 'app.server.log')
        formatter = logging.Formatter(fmt)
        handler = TimedRotatingFileHandler(filename, when="midnight", interval=1)
        handler.suffix = '%Y-%m-%d'
        handler.setFormatter(formatter)
        logger = logging.getLogger('app.server')
        logger.addHandler(handler)
        logger.setLevel(logging.INFO)
        return logger
    else:
        filename = os.path.join(package_dir, 'app.client.log')
        loggername = 'app.client'
        logging.basicConfig(
            format=fmt,
            filename=filename,
            level=logging.INFO,
        )
        return logging.getLogger(loggername)


def log(logger):
    def decorator(func):
        @wraps(func)
        def call(*args, **kwargs):
            logger.info('%s %s: Function called', func.__module__, func.__name__)
            try:
                return func(*args, **kwargs)
            except:
                logger.exception('%s %s: Exception: ', func.__module__, func.__name__)
                raise

        return call
    return decorator



from sqlalchemy import Table, Column, Integer, String, MetaData, DateTime, ForeignKey, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import os
Base = declarative_base()


class Client(Base):
    __tablename__ = 'clients'
    id = Column(Integer, primary_key=True)
    login = Column(String)
    info = Column(String)

    def __init__(self, login, info):
        self.login = login
        self.info = info

    def __repr__(self):
        return '<Client({0}, {1})>'.format(self.login, self.info)


class ClientHistory(Base):
    __tablename__ = 'client_history'
    id = Column(Integer, primary_key=True)
    login_time = Column(DateTime)
    ip = Column(String)
    clientid = Column(Integer, ForeignKey('clients.id'))

    def __init__(self, login_time, ip):
        self.login_time = login_time
        self.ip = ip

    def __repr__(self):
        str_log_in_at = self.login_time.strftime("%d-%m-%Y %H:%M:%S")
        return '<Client {0} logged in at {1}>'.format(self.ip, str_log_in_at)


class ContactList(Base):
    __tablename__ = 'contacts'
    id = Column(Integer, primary_key=True)
    ownerid = Column(Integer, ForeignKey('clients.id'))
    contactid = Column(Integer, ForeignKey('clients.id'))

    def __init__(self, ownerid, contactid):
        self.ownerid = ownerid
        self.contactid = contactid


class ServerDB:
    def __init__(self):
        package_dir = os.path.abspath(os.path.dirname(__file__))
        db_dir = os.path.join(package_dir, 'server.sqlite?check_same_thread=False')
        sqlite_path = ''.join(['sqlite:///', db_dir])
        self.engine = create_engine(sqlite_path)
        Base.metadata.create_all(self.engine)
        Session = sessionmaker(bind=self.engine)
        self.session = Session()


def mainloop():
    serverdb = ServerDB()
    print('Session:', serverdb.session)
    test_client = Client('testlogin', 'testinfo')
    dany_client = Client('Dany', 'admin')
    contact = ContactList(2, 1)
    name = 'Dany'
    # serverdb.session.add(test_client)
    # serverdb.session.add(dany_client)
    # serverdb.session.add(contact)
    # serverdb.session.commit()
    q_client = serverdb.session.query(Client).filter_by(login=name).first()
    print('Query:', q_client)


if __name__ == '__main__':
    mainloop()


# coding: utf-8
from queue import Queue
from socket import *
from threading import Thread

from aemessenger.Log.log_config import configlogging

# from aemessenger.OLD.jimclient import get_presence_msg, get_user_to_chat_msg

logger = configlogging(False)
# ADDRESS = 'localhost'
# PORT = 7777
#
#
# @log(logger)
# def checkcmdargs():
#     args = [0, 0]
#     args[0] = argv[1] if len(argv) > 1 else ADDRESS
#     args[1] = int(argv[2]) if len(argv) > 2 else PORT
#     return args


class SendThread(Thread):
    def __init__(self, client):
        Thread.__init__(self)
        self.client = client
        self.is_stopped = False
        self.setDaemon(True)

    def run(self):
        while not self.is_stopped:
            msg = self.client.send_queue.get()
            print('sending:', msg)
            self.client.sock.send(msg.utf8)
            self.client.send_queue.task_done()


class RecieveThread(Thread):
    def __init__(self, client):
        Thread.__init__(self)
        self.client = client
        self.is_stopped = False
        self.setDaemon(True)

    def run(self):
        while not self.is_stopped:
            resp = self.client.sock.recv(1024)
            if resp:
                resp_json = resp.decode('utf-8')
                for r in resp_json.split('\r\n\r\n'):
                    if r == '':
                        continue
                    print('recieved:', r)
                    if 'msg' in r:  # Если это сообщение от пользователя/чата
                        self.client.msg_queue.put(r)
                    else:  # служебное сообщение сервера
                        self.client.resp_queue.put(r)


class SocketClient(object):
    def __init__(self, args):
        self.sock = socket(AF_INET, SOCK_STREAM)
        self.address, self.port = args
        self.sock.connect((self.address, self.port))
        self.send_queue = Queue()  # отправка сообщений на сервер
        self.resp_queue = Queue()  # сервисные сообщения от сервера
        self.msg_queue = Queue()  # сообщения пользователей/чатов с сервера
        self.is_alive = True  # TODO: использовать

        self.send_thread = SendThread(self)
        self.recv_thread = RecieveThread(self)

        self.send_thread.start()
        self.recv_thread.start()
        # self.start_async_tasks()

    def add_to_send_queue(self, msg):
        self.send_queue.put(msg)

    # async def get_responses(self):
    #     print('started')
    #     while True:
    #         resp = await self.sock.recv(1024)
    #         if resp:
    #             resp_json = resp.decode('utf-8')
    #             for r in resp_json.split('\r\n\r\n'):
    #                 if r == '':
    #                     continue
    #                 print('recieved:', r)
    #                 if 'msg' in r:  # Если это сообщение от пользователя/чата
    #                     self.msg_queue.put(r)
    #                 else:  # служебное сообщение сервера
    #                     self.resp_queue.put(r)
    #
    # async def send_messages(self):
    #     while True:
    #         msg = self.send_queue.get()
    #         print('sending:', msg)
    #         await self.sock.send(msg.utf8)
    #         self.send_queue.task_done()
    #
    # def start_async_tasks(self):
    #     eloop = asyncio.get_event_loop()
    #     asyncio.run_coroutine_threadsafe(self.send_messages(), eloop)
    #     asyncio.run_coroutine_threadsafe(self.get_responses(), eloop)

# if __name__ == '__main__':
#     client_test = SocketClient()
#     client_test.send_queue.put(get_presence_msg())
#
#     while client_test.is_alive:
#         text = input("Введите сообщение: ")
#         if text == 'quit':
#             break
#         client_test.send_queue.put(get_user_to_chat_msg(chatname='all', text=text))

# tm = self.sock.recv(1024)
# print(parse_server_message(tm.decode('ascii')))
# if mode == '-r':
#     while True:
#         tm = s.recv(1024)
#         if tm:
#             print(tm.decode('ascii'))  # пока никак не парсим, еще не утрясли кодирование протокола
# if mode == '-w':
#     while True:
#         text = input("Введите сообщение: ")
#         if text == 'quit':
#             break
#         s.send(get_user_to_chat_msg(chatname='all', text=text).encode('ascii'))
# self.sock.close()




import datetime
import json
import locale
from sys import argv
from aemessenger.Client.clientstorage import ClientDB, MsgHistory
from aemessenger.Client.socketclient import SocketClient
from aemessenger.JIM.jimmsg import JIMPresenceMsg, JIMGetContactsMsg, JIMAddContactMsg, JIMDelContactMsg, JIMUserMsg
from aemessenger.JIM.jimresponse import JIMResponse

ADDRESS = 'localhost'
PORT = 7777


class ClientController(object):
    def __init__(self, username):
        self.db = ClientDB(username)
        self.args = self.checkcmdargs()
        try:
            self.client = SocketClient(self.args)
        except ConnectionRefusedError as e:
            print(e)
            raise e
        self.username = username
        self.password = ''
        self.status = ''

    def checkcmdargs(self):
        args = [0, 0]
        args[0] = argv[1] if len(argv) > 1 else ADDRESS
        args[1] = int(argv[2]) if len(argv) > 2 else PORT
        return args

    def send_presense(self):
        if self.username == '':
            self.username = 'Guest'
        msg = JIMPresenceMsg(self.username, self.status)
        self.client.add_to_send_queue(msg)
        action = self.parse_server_response()
        print(action)
        if action.action == 'quit':
            return -1
        return 0

    def parse_server_response(self):
        resp_json = self.client.resp_queue.get()

        if 'response' in resp_json:
            resp = JIMResponse.fromjson(resp_json)
            if hasattr(resp, 'quantity'):  # Сервер говорит, сколько будет контактов
                return ClientAction('quantity', resp.quantity)
            elif resp.response == '200':
                return ClientAction(True, None)
            elif resp.response == '409':  # Имя не подходит
                return ClientAction('quit', None)
            elif resp.response == '404':  # Что-то не нашли, например сообщение не доставлено
                return ClientAction(False, None)
        elif 'action' in resp_json:
            resp = json.loads(resp_json)  # dict, можно реализовать from_json в jimmsg
            if resp['action'] == 'contact_list':
                return ClientAction('contact_list', resp['user_id'])
        self.client.resp_queue.task_done()  # TODO: поправить, программа сюда не заходит

    def get_contacts(self):
        msg = JIMGetContactsMsg(self.username)
        self.client.add_to_send_queue(msg)
        # Получить контакты с сервера
        quantity = -1
        while quantity != 0:
            action = self.parse_server_response()  # None если клиента нет в базе
            if action.action:
                print(action)
                if action.action == 'quantity':
                    quantity = int(action.value)
                elif action.action == 'contact_list':
                    self.db.add_contact(action.value)
                    quantity -= 1
            else:
                return -1
            self.client.resp_queue.task_done()
        return 0

    def add_contact(self, contact_name):
        msg = JIMAddContactMsg(contact_name, self.username)
        self.client.add_to_send_queue(msg)
        action = self.parse_server_response()
        if action.action is not True:
            pass  # TODO обработать ошибку
        self.get_contacts()

    def del_contact(self, contact_name):
        msg = JIMDelContactMsg(contact_name, self.username)
        self.client.add_to_send_queue(msg)
        action = self.parse_server_response()
        if action.action is not True:
            pass  # TODO обработать ошибку
        self.get_contacts()

    def send_user_msg(self, message, to):
        msg = JIMUserMsg(to, self.username, message)
        self.client.add_to_send_queue(msg)
        action = self.parse_server_response()
        if action.action is not True:
            return -1  # TODO обработать ошибку
        self.add_user_msg_to_db(msg)  # добавить сообщение в базу
        return 0

    def add_user_msg_to_db(self, jimmsg):
        locale.setlocale(locale.LC_ALL, 'en_US')
        time = datetime.datetime.strptime(jimmsg.time, "%a %b %d %H:%M:%S %Y")
        msg = MsgHistory(None, jimmsg.to, jimmsg.account, jimmsg.message, time)
        self.db.add_msg_history(msg)


class ClientAction(object):
    """Объект класса говорит, что делать клиенту на основе ответов сервера"""
    def __init__(self, action, value):
        self.action = action  # что делать
        self.value = value  # с чем делать

    def __repr__(self):
        return str(self.action) + ': ' + str(self.value)


def mainloop():
    controller = ClientController('Dany')
    if controller.send_presense() < 0:
        return
    controller.get_contacts()
    controller.send_user_msg('hi', 'Dany')
    while True:
        print(controller.client.msg_queue.get())
        controller.client.msg_queue.task_done()

if __name__ == '__main__':
    mainloop()



import sys
import textwrap
import os
from aemessenger.Client.clientcontroller import ClientController
from PyQt5 import QtWidgets, uic, QtGui
from PyQt5.QtCore import Qt, QThread, pyqtSignal
from PyQt5.QtGui import QStandardItem, QStandardItemModel, QIcon
from PyQt5.QtWidgets import QWidget, QMessageBox, QLabel, QVBoxLayout, QStackedLayout, QHBoxLayout
from aemessenger.UI.add_contact_dialog import AddContactDialog
from aemessenger.UI.username_dialog import UsernameDialog
from aemessenger.JIM.jimmsg import JIMUserMsg, JIMMessageBuilder, JIMChatMsg


class ClientWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.package_dir = os.path.abspath(os.path.dirname(__file__))
        ui_path = os.path.join(self.package_dir, 'UI', 'client.ui')
        self.window = uic.loadUi(ui_path)
        self.contacts = QStandardItemModel(self.window.contactListView)
        self.controller = None  # контроллер клиента
        self.msg_thread = None  # поток обработки входящих сообщений

        # список лэйаутов чата вида 'contact':[index, QVBoxLayout].
        # Каждому контакту свой лэйаут
        self.chat_stackedLayout = QStackedLayout()
        self.chat_layout_indexes = {}

        self.vbar = self.window.chat_scroll.verticalScrollBar()
        self.init_ui()

    def init_ui(self):
        self.window.contactListView.setModel(self.contacts)
        self.window.contactListView.clicked.connect(self.contact_selected)
        self.window.contactListView.setAttribute(Qt.WA_MacShowFocusRect, 0)  # Убрать стремный фокус

        self.window.addContactButton.clicked.connect(self.add_contact)
        self.window.sendButton.clicked.connect(self.send_msg)

        self.window.chat_scroll_widget.setLayout(self.chat_stackedLayout)
        self.window.chat_scroll_widget.setContentsMargins(20, 20, 25, 20)

        self.window.show()

    def get_contacts(self):
        self.contacts.clear()
        result = self.controller.get_contacts()
        if result < 0:
            show_error_msg('Contacts not found', QMessageBox.Warning)
            return
        names = self.controller.db.get_contacts()
        icon_path = os.path.join(self.package_dir, 'Img', 'user.png')
        for name in names:
            item = QStandardItem(name)
            item.setEditable(False)
            font = item.font()
            font.setPointSize(15)
            item.setFont(font)
            item.setIcon(QIcon(icon_path))
            self.contacts.appendRow(item)
        self.contacts.sort(0)
        self.fill_chat_layouts(names)

    def add_contact(self):
        contact, result = AddContactDialog().get_username()
        if contact is '' or result is False:
            return
        self.controller.add_contact(contact)
        self.get_contacts()

    def fill_chat_layouts(self, names):
        index = 0
        for name in names:
            if name in self.chat_layout_indexes:  # добавляем контакт, обновляем индекс
                self.chat_layout_indexes[name][0] = index
                index += 1
                continue
            chatWidget = QWidget()
            layout = QVBoxLayout()
            layout.addStretch(1)
            layout.setContentsMargins(0, 0, 0, 0)
            chatWidget.setLayout(layout)
            self.chat_stackedLayout.addWidget(chatWidget)
            self.chat_layout_indexes[name] = [index, layout]
            index += 1

    def contact_selected(self, modelindex):
        # Поменять заголовок при клике на контакт (nameLabel)
        name = str(self.window.contactListView.model().itemData(modelindex)[0])
        self.window.nameLabel.setText(name)
        i = self.chat_layout_indexes[name][0]
        self.chat_stackedLayout.setCurrentIndex(i)

    def send_msg(self):
        index = self.window.contactListView.selectionModel().selectedIndexes()
        # Выбран ли контакт?
        if len(index) is 0:
            return
        # Есть ли текст сообщения?
        text = self.window.textEdit.toPlainText()
        if not text:
            return
        name = self.window.contactListView.model().itemData(index[0])[0]
        if name != self.controller.username:
            self.add_gui_msg(name, text, False)  # добавить в gui
        result = self.controller.send_user_msg(text, name)
        # Успешно отправлено?
        if result == 0:
            self.window.textEdit.clear()
        else:
            show_error_msg('Not delivered', QMessageBox.Warning)

    def add_gui_msg(self, name, text, is_left):
        msg = MsgBubble(text, is_left)
        if is_left:
            self.chat_layout_indexes[name][1].addWidget(msg)
        else:
            # spacer прижмет сбоку сообщение слева для случая, когда надо сделать пузырь справа
            # а т.к. общий лэйаут вертикальный, то мы засунем spacer и пузырь
            # в горизонтальный лэйаут, а лэйаут в QWidget
            group_line = QWidget()
            horizontal_layout = QHBoxLayout()
            horizontal_layout.addStretch(1)
            spacer = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
            horizontal_layout.addItem(spacer)
            horizontal_layout.addWidget(msg)
            horizontal_layout.setContentsMargins(0, 0, 0, 0)
            group_line.setLayout(horizontal_layout)
            self.chat_layout_indexes[name][1].addWidget(group_line)
        # Скролл до упора вниз, но что-то не очень работает
        h = self.window.chat_scroll.height()
        self.vbar.setValue(h*10000)

    def on_msg_recieved(self, jimmsg):
        if type(jimmsg) is JIMUserMsg:
            left = True if jimmsg.to == self.controller.username else False
            self.add_gui_msg(jimmsg.account, jimmsg.message, left)
        elif type(jimmsg) is JIMChatMsg:
            raise NotImplemented


class MsgThread(QThread):
    msg_ready = pyqtSignal(object)

    def __init__(self, client_widget, queue):
        QThread.__init__(self)
        self.client = client_widget
        if not queue:
            self.isAlive = False
        self.queue = queue

    def run(self):
        while True:
            msg = self.queue.get()
            if msg:
                print('recieved msg:', msg)
                jimmsg = JIMMessageBuilder.get_msg_from_json(msg)
                self.msg_ready.emit(jimmsg)
                self.queue.task_done()


class MsgBubble(QLabel):
    """Красивый пузырь для сообщений.
    При is_left == True, покрасится в зеленый"""
    def __init__(self, text, is_left):
        text = textwrap.fill(text, 20)  # разбить текст по 20 знаков, чтобы он влез в пузырь
        super(MsgBubble, self).__init__(text)
        self.is_left = is_left
        width = len(text) * 10 if len(text) < 20 else 200  # Если текст не очень большой, можно сделать пузырь поменьше
        self.setMaximumWidth(width + 10)
        self.setMinimumWidth(width)
        self.setWordWrap(True)
        self.setContentsMargins(5, 5, 5, 5)

    def paintEvent(self, e):
        p = QtGui.QPainter(self)
        p.setRenderHint(QtGui.QPainter.HighQualityAntialiasing, True)
        if self.is_left:
            p.setBrush(Qt.green)
        p.drawRoundedRect(0, 0, self.width()-1, self.height()-1, 8, 8)

        super(MsgBubble, self).paintEvent(e)


def show_error_msg(text, status):
    msg = QMessageBox()
    msg.setText(text)
    msg.setIcon(status)
    msg.setStandardButtons(QMessageBox.Ok)
    msg.exec_()


def mainloop():
    app = QtWidgets.QApplication(sys.argv)
    client = ClientWidget()

    # Получить username
    username, result = UsernameDialog.get_username()
    if username == '' or result is False:
        username = 'Guest'
    print(username)
    title = 'Messenger - ' + username
    print(title)
    client.window.setWindowTitle(title)

    # Создать контроллер
    try:
        client.controller = ClientController(username)
    except ConnectionRefusedError as e:
        show_error_msg('Server is not available', QMessageBox.Critical)
        sys.exit()

    # Отправить presense
    if client.controller.send_presense() < 0:
        show_error_msg('Username already exists', QMessageBox.Critical)
        sys.exit()

    # Подключить поток обработки входящих сообщений msg
    client.msg_thread = MsgThread(client, client.controller.client.msg_queue)
    client.msg_thread.msg_ready.connect(client.on_msg_recieved)
    client.msg_thread.finished.connect(app.exit)
    client.msg_thread.start()

    # Получить список контактов
    client.get_contacts()

    sys.exit(app.exec_())

if __name__ == '__main__':
    mainloop()


import socket
import time
import os
from socketserver import BaseRequestHandler
from socketserver import TCPServer, ThreadingMixIn
from sys import argv
from aemessenger.Server.servercontroller import JIMServerController
from aemessenger.JIM.jimresponse import JIMResponse


class ThreadingJIMServer(ThreadingMixIn, TCPServer):
    def __init__(self, server_address, request_handler):
        super().__init__(server_address, request_handler, True)
        self.clients = list()
        self.usernames = list()
        self.controller = JIMServerController()

    def add_client(self, client, username):
        self.clients.append(client)
        self.usernames.append(username)

    def remove_client(self, client):
        index = self.clients.index(client)
        del self.clients[index]
        del self.usernames[index]

    def get_client_by_username(self, username):
        index = self.usernames.index(username)
        return self.clients[index]

    def get_username_by_client(self, client):
        index = self.clients.index(client)
        return self.usernames[index]

    allow_reuse_address = True
    max_children = 100


class JIMRequestHandler(BaseRequestHandler):
    def setup(self):
        """
        Подключить нового клиента и узнать его имя.
        Клиент после подключения должен сразу отправить presense-сообщение
        """
        print('Client connected:', self.client_address)
        presense_data = self.request.recv(1024)  # получить имя
        response, action = self.server.controller.parse_client_message(presense_data.decode('utf-8'), None)
        if action.usernames[0] in self.server.usernames:
            uid = response.uid
            response = JIMResponse(409, 'Username already exists', None, None, uid)
            print('Error:', self.client_address, 'tried to use username:', action.usernames[0])
            self.server.add_client(self, action.usernames[0] + uid)
        else:
            print(self.client_address, 'got username:', action.usernames[0])
            self.server.add_client(self, action.usernames[0])  # записать имя и клиента
        self.request.sendall(response.utf8)  # отправить ответ

    def handle(self):
        if isinstance(self.request, socket.socket):
            # работа с потоком
            try:
                while True:
                    data = self.request.recv(1024)
                    if not data:
                        print('{0} disconnected'.format(self.client_address))
                        break
                    print(data.decode('utf-8'))
                    response, action = self.server.controller.parse_client_message(data.decode('utf-8'),
                                                                                   self.server.get_username_by_client(self))
                    if action:
                        if action.action == 'quantity':  # Шлём контакты
                            for r in response:
                                self.request.sendall(r.utf8 + '\r\n\r\n'.encode('utf-8'))
                        elif action.action == 'presence':  # Отвечаем на презенс
                            self.request.sendall(response.utf8)
                        elif action.action == 'msg':  # рассылаем сообщения
                            for username in action.usernames:
                                try:
                                    client = self.server.get_client_by_username(username)
                                    client.request.sendall(action.msg.encode('utf-8'))
                                except ValueError as e:
                                    # Не нашли товарища онлайн
                                    uid = response.uid
                                    response = JIMResponse(404, 'Username not found online', None, None, uid)
                            self.request.sendall(response.utf8)  # TODO если сообщение для чатика, то вылетит эксепшн
                    else:
                        self.request.sendall(response.utf8)
            except Exception as e:
                print()
                print('{0} expected an error'.format(self.client_address))
                raise e

    def finish(self):
        self.server.remove_client(self)


def checkcmdargs():
    args = ['localhost', 7777]
    args[0] = argv[1] if len(argv) > 1 else 'localhost'
    args[1] = int(argv[2]) if len(argv) > 2 else 7777
    return args


def mainloop():
    address, port = checkcmdargs()
    serv = ThreadingJIMServer((address, port), JIMRequestHandler)
    print('Сервер запущен в {0}'.format(time.ctime(time.time())))
    serv.serve_forever()

if __name__ == '__main__':
    mainloop()



from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QLineEdit, QDialogButtonBox


class UsernameDialog(QDialog):
    def __init__(self, parent=None):
        super(UsernameDialog, self).__init__(parent)

        self.setWindowTitle('Enter username')
        layout = QVBoxLayout(self)

        # поле для имени
        self.username_edit = QLineEdit(self)
        layout.addWidget(self.username_edit)

        # OK and Cancel buttons
        buttons = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
            Qt.Horizontal, self)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    # Получить имя из диалога
    def username(self):
        return self.username_edit.text()

    # Создать диалог и вернуть юзернейм
    @staticmethod
    def get_username(parent=None):
        dialog = UsernameDialog(parent)
        result = dialog.exec_()
        name = dialog.username()
        return name, result == QDialog.Accepted



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='aemessenger',
    version='0.1',
    description='A sample Python socket messenger',
    long_description='A Geekbrains.ru messenger graduation project',
    url='https://github.com/danyae/PythonMessenger',
    author='Dany ae',
    author_email='noreply@aetech.ru',
    license='GPL',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Communications :: Chat :: Internet Relay Chat',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='sample messenger chat socket geekbrains',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['*OLD*', '*Tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'PyQt5>=5.9',
        'SQLAlchemy>=1.1.14'
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    # extras_require={
    #     'dev': ['check-manifest'],
    #     'test': ['coverage'],
    # },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        # 'Img': ['*.png', '*.jpg'],
        # 'UI': ['client.ui']
        '': ['*']
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('db', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'client=aemessenger.gui_client:mainloop',
            'server=aemessenger.server:mainloop'
        ],
    },
)


import json
import time
from uuid import uuid1


# actions: authenticate, presence, quit, msg, join, leave, probe (server)
# fields: action, time, user (account_name, status), type, to, from, encoding, message, room
class JIMMsg:
    def __init__(self, action):
        self.action = action
        self.time = time.ctime(time.time())
        self.uid = str(uuid1())

    @property
    def json(self):
        fields = self.__dict__
        j = {c: fields[c] for c in fields if fields[c] is not None}
        return json.dumps(j)

    def from_dict(self, values):
        for k, v in values.items():
            setattr(self, k, v)
        return self

    @property
    def utf8(self):
        return self.json.encode('utf-8')


class JIMAuthMsg(JIMMsg):
    def __init__(self, account_name='', password=''):
        super().__init__('authenticate')
        self.user = {'account_name': account_name, 'password': password}


class JIMPresenceMsg(JIMMsg):
    def __init__(self, account_name='', status='', msgtype=None):
        super().__init__('presence')
        self.user = {'account_name': account_name, 'status': status}
        self.type = msgtype


class JIMProbeMsg(JIMMsg):
    def __init__(self):
        super().__init__('probe')


class JIMQuitMsg(JIMMsg):
    def __init__(self):
        super().__init__('quit')


class JIMUserMsg(JIMMsg):
    def __init__(self, to='', acc_name='', message='', encoding='utf-8'):
        super().__init__('msg')
        self.to = to
        self.account = acc_name  # from - служебное слово
        self.encoding = encoding
        self.message = message


class JIMChatMsg(JIMMsg):
    def __init__(self, to='', acc_name='', message=''):
        super().__init__('msg')
        self.to = '#' + to
        self.account = acc_name  # from - служебное слово
        self.message = message


class JIMJoinChatMsg(JIMMsg):
    def __init__(self, room_name=''):
        super().__init__('join')
        self.room = '#' + room_name


class JIMLeaveChatMsg(JIMMsg):
    def __init__(self, room_name=''):
        super().__init__('leave')
        self.room = '#' + room_name


class JIMGetContactsMsg(JIMMsg):
    def __init__(self, user_id):
        super().__init__('get_contacts')
        self.user_id = user_id


class JIMSendContactListMsg(JIMMsg):
    def __init__(self, user_id):
        super().__init__('contact_list')
        self.user_id = user_id  # nickname


class JIMAddContactMsg(JIMMsg):
    def __init__(self, contact_id, user_id):
        super().__init__('add_contact')
        self.contact_id = contact_id  # nickname
        self.user_id = user_id


class JIMDelContactMsg(JIMMsg):
    def __init__(self, contact_id, user_id):
        super().__init__('del_contact')
        self.contact_id = contact_id  # nickname
        self.user_id = user_id


class JIMMessageBuilder:
    msg_classes = {'authenticate': 'JIMAuthMsg', 'presence': 'JIMPresenceMsg',
                   'quit': 'JIMQuitMsg', 'msg': ('JIMUserMsg', 'JIMChatMsg'),
                   'join': 'JIMJoinChatMsg', 'leave': 'JIMLeaveChatMsg',
                   'probe': 'JIMProbeMsg', 'get_contacts': 'JIMGetContactsMsg',
                   'contact_list': 'JIMSendContactListMsg', 'add_contact': 'JIMAddContactMsg',
                   'del_contact': 'JIMDelContactMsg'
                   }

    @staticmethod
    def get_msg_from_json(json_msg):
        # Возвращает объект соответствующего класса сообщения в зависимости от action
        try:
            parsed_msg = json.loads(json_msg)  # Получить словарь атрибутов
        except ValueError:
            return None
        action = parsed_msg['action']
        if action == 'msg':  # сообщение пользователю или в чат
            if parsed_msg['to'][0] == '#':
                class_ = globals()[JIMMessageBuilder.msg_classes[action][1]]
            else:
                class_ = globals()[JIMMessageBuilder.msg_classes[action][0]]
        else:
            class_ = globals()[JIMMessageBuilder.msg_classes[action]]
        return class_().from_dict(parsed_msg)

if __name__ == '__main__':
    msg = JIMUserMsg('admin', 'user', 'hey man')
    js = msg.json
    print(js)
    msg = JIMMessageBuilder.get_msg_from_json(js)
    print(msg)
    print(msg.json)
    print(msg.utf8)


from sqlalchemy import Table, Column, Integer, String, MetaData, DateTime, ForeignKey, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import os


Base = declarative_base()


class ContactList(Base):
    __tablename__ = 'contacts'
    id = Column(Integer, primary_key=True)
    contactid = Column(Integer)
    username = Column(String)
    info = Column(String)

    def __init__(self, contactid, username, info):
        self.contactid = contactid
        self.username = username
        self.info = info

    def __repr__(self):
        return '<Contact({0}, {1})>'.format(self.username, self.info)


class MsgHistory(Base):
    __tablename__ = 'msghistory'
    id = Column(Integer, primary_key=True)
    chatid = Column(Integer, nullable=True)
    to_username = Column(String, nullable=True)
    from_username = Column(String)
    msg = Column(String)
    timestamp = Column(DateTime)

    def __init__(self, chatid, to_username, from_username, msg, timestamp):
        self.chatid = chatid
        self.to_username = to_username
        self.from_username = from_username
        self.msg = msg
        self.timestamp = timestamp

    def __repr__(self):
        str_timestamp = self.timestamp.strftime("%d-%m-%Y %H:%M:%S")
        if self.chatid:
            return '<MSG at{0} in chat {1}, user {2}: {3}'.format(
                str_timestamp, self.chatid, self.from_username, self.msg)
        else:
            return '<MSG at{0}, to user {1} from {2}: {3}'.format(
                str_timestamp, self.to_username, self.from_username, self.msg)


class ClientDB:
    def __init__(self, username):
        package_dir = os.path.abspath(os.path.dirname(__file__))
        db_dir = os.path.join(package_dir, 'client' + username + '.sqlite?check_same_thread=False')
        sqlite_path = ''.join(['sqlite:///', db_dir])
        # self.engine = create_engine('sqlite:///client' + username + '.sqlite?check_same_thread=False')
        self.engine = create_engine(sqlite_path)
        Base.metadata.create_all(self.engine)
        Session = sessionmaker(bind=self.engine)
        self.session = Session()

    def add_contact(self, username):
        exist = self.session.query(ContactList).filter_by(username=username).first()
        if not exist:
            contact = ContactList(username, username, '')
            self.session.add(contact)
            self.session.commit()

    def get_contacts(self):
        clients = self.session.query(ContactList).all()
        return [c.username for c in clients]

    def add_msg_history(self, msg):
        self.session.add(msg)
        self.session.commit()

def mainloop():
    clientdb = ClientDB('Dany')
    print('Session:', clientdb.session)


if __name__ == '__main__':
    mainloop()
